<?php

namespace Drupal\ww_publish;

use Drupal\field\Entity\FieldConfig;
use Drupal\paragraphs\Entity\Paragraph;

class ParagraphField {

  /**
   * Article Data.
   *
   * @var \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList
   */
  private $fieldConfig;

  /**
   * Article Data.
   *
   * @var Object
   */
  private $articleData;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $field_config
   *   Field config.
   * @param Object $article_data
   *   Article data.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($field_config, $article_data, $logger) {
    $this->fieldConfig = $field_config;
    $this->articleData = $article_data;
    $this->logger = $logger;
  }

  /**
   * Get article data.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getArticleData() {

    $identifiers = $this->getIdentifiers();
    $this->logger->info('Identifiers: <pre><code>' . print_r($identifiers, TRUE) . '</code></pre>');

    $articleData = [];

    foreach ($this->articleData->data->content as $contentStationComponent) {
      $identifierKey = array_search($contentStationComponent->identifier, array_column($identifiers, 'identifier'));

      if (is_int($identifierKey)) {
        $identifier = $identifiers[$identifierKey];

        // Get content from the text property.
        $content = '';
        foreach ($contentStationComponent->content->text as $text) {
          $attributeContent = $text->insert;
          if (property_exists($text, 'attributes')) {
            foreach ($text->attributes as $attribute => $attributeValue) {
              $template = new Template($attribute, 'attribute', $this->logger);
              if ($template->exist()) {
                $attributeContent = $template->apply([
                  'insert' => $attributeContent,
                  $attribute => $attributeValue,
                ]);
              }
            }
          }
          $content .= $attributeContent;
        }

        // Apply a template on the content.
        $template = new Template($identifier['identifier'], 'component', $this->logger);
        if ($template->exist()) {
          $content = $template->apply(['insert' => $content]);
        }

        // Create a new paragraph.
        $paragraphData = [
          'type' => $identifier['paragraph_type'],
          $identifier['field_name'] => $content,
        ];
        $paragraph = Paragraph::create($paragraphData);
        $paragraph->{$identifier['field_name']}->format = 'full_html';
        $paragraph->save();

        $articleData[] = [
          'target_id' => $paragraph->id(),
          'target_revision_id' => $paragraph->getRevisionId(),
        ];
      }
    }

    $this->logger->info('Paragraph article data: <pre><code>' . print_r($articleData, TRUE) . '</code></pre>');
    return $articleData;
  }

  /**
   * Get all defined Content Station identifiers.
   *
   * All identifiers from fields in all enabled paragraph types is returned as
   * an array, of arrays with the following keys: identifier, paragraph_type,
   * field_name.
   *
   * @return array
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function getIdentifiers() {
    $identifiers = [];
    // Get all enabled paragraph types.
    $enabledParagraphTypes = $this->fieldConfig->get('settings')['handler_settings']['target_bundles'];
    foreach ($enabledParagraphTypes as $type) {
      $fields = \Drupal::service('entity.manager')->getFieldDefinitions('paragraph', $type);
      foreach ($fields as $field_name => $field) {
        if ($field instanceof FieldConfig && $field->get('entity_type') === 'paragraph') {
          $identifier = $field->getThirdPartySetting('ww_publish', 'content_station_identifier');
          $paragraph_type = $field->get("bundle");
          $identifiers[] = [
            'identifier' => $identifier,
            'paragraph_type' => $paragraph_type,
            'field_name' => $field_name,
          ];
        }
      }
    }
    return $identifiers;
  }
}
